/***********************************************************
 * resample.c
 *    Refer to resample.m for documentation.  This C version
 *    is about 10x faster.
 *
 * Author: David Ross, Nov. 27, 2003.
 * $Id: resample.c,v 1.1 2006/09/14 15:44:08 dross Exp $
 ***********************************************************/

#include "mex.h"
#include <math.h>

/* return the pixel value at (x,y) in the image */
#define imgval(x,y) image[(int)((x) * image_rows + (y))]


/***********************************************************
  Min and Max.  Either the macro or function versions can
  be used.  The functions don't seem to be any slower than
  the macros.
 ***********************************************************/

/* #define dmax(x,y) (((x)>(y)) ? (x) : (y)) */
/* #define dmin(x,y) (((x)>(y)) ? (y) : (x)) */

double dmax(double x, double y) {
   if (x>y) {
      return x;
   } else {
      return y;
   }
}

double dmin(double x, double y) {
   if (x>y) {
      return y;
   } else {
      return x;
   }
}


void resample(double *image, int image_rows, int image_cols, 
	      double *x, double *y, int Nsamples, double *result);


/***********************************************************
 mexFunction - parse & check arguments, etc.
 ***********************************************************/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, 
		 const mxArray *prhs[]) {
   int Nx, Ny, image_rows, image_cols;
   double *image, *x, *y, *result;

   /* check for the proper number of arguments */
   if (nrhs != 3) {
      mexErrMsgTxt("wrong number of input arguments");
   }
   if (nlhs != 1) {
      mexErrMsgTxt("wrong number of output arguments");
   }

   /* get sizes of x and y*/
   image_rows = mxGetM(prhs[0]);
   image_cols = mxGetN(prhs[0]);
   Ny = mxGetM(prhs[1]) * mxGetN(prhs[1]);
   Nx = mxGetM(prhs[2]) * mxGetN(prhs[2]);

   /* make sure x and y are the same size */
   if (Nx != Ny) {
      mexErrMsgTxt("length of x must equal size of y");
   }

   /* make sure all the input arguments are doubles */
   if (!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1]) || ! mxIsDouble(prhs[2])) {
      mexErrMsgTxt("the input arguments must all be of type double");
   }

   /* get image, x, and y */
   image = mxGetPr(prhs[0]);
   x = mxGetPr(prhs[1]);
   y = mxGetPr(prhs[2]);

   /* create the result */
   plhs[0] = mxCreateDoubleMatrix(Nx, 1, mxREAL);
   result = mxGetPr(plhs[0]);

   /* do it */
   resample(image, image_rows, image_cols, x, y, Nx, result);
}


/***********************************************************
 resample - do the resampling
 Note: This version isn't the 'proper' way to do it, if 
   subsampling by more than a factor of maybe 2x.
 ***********************************************************/
void resample(double *image, int image_rows, int image_cols, 
	      double *x, double *y, int Nsamples, double *result) {
   int i;
   int x0,y0;
   double dx,dy;

   for(i=0; i < Nsamples; i++) {

      /* need to add checks here incase x < 1 or x > image_cols */
      if (x[i] < 1) {
	 x[i] = 1;
      } else if (x[i] > image_cols) {
	 x[i] = image_cols;
      }
      x0 = (int) floor(x[i]);
      dx = x[i] - x0;
      x0 -= 1; /* matlab arrays start at 1, but C arrays at 0 */

      /* need to add checks here incase y < 1 or y > image_rows */
      if (y[i] < 1) {
	 y[i] = 1;
      } else if (y[i] > image_rows) {
	 y[i] = image_rows;
      }
      y0 = (int)floor(y[i]);
      dy = y[i] - y0;
      y0 -= 1; /* matlab arrays start at 1, but C arrays at 0 */

      result[i] = imgval(x0,y0) * (1-dx) * (1-dy) 
	       + imgval(x0,y0+1) * (1-dx) * dy
	       + imgval(x0+1,y0) * dx * (1-dy)
	       + imgval(x0+1,y0+1) * dx * dy;
   }
}
